/* $Id: petsctoolfe.cpp,v 1.21 2001/06/04 22:58:33 buschelm Exp $ */
#include <Windows.h>
#include "petsctoolfe.h"

using namespace PETScFE;

tool::tool(void) {
  tool::OptionTags= "hnpuvw";
  tool::Options['h'] = &tool::FoundHelp;
  tool::Options['n'] = &tool::FoundNoDetect;
  tool::Options['p'] = &tool::FoundPath;
  tool::Options['u'] = &tool::FoundUse;
  tool::Options['v'] = &tool::FoundV;
  tool::Options['w'] = &tool::FoundWoff;
  tool::Options[UNKNOWN] = &tool::FoundUnknown;
  version_string  = "PETSc's Win32 Front End, version 1.1.1";
#if defined(WINDOWS_NT4_OS)
  version_string  += "-NT4";
#endif
  detect       = true;
  woff         = false;
  verbose      = false;
  helpfound    = false;
  versionfound = false;
  inpath       = false;
}
  
void tool::GetArgs(int argc,char *argv[]) {
  for (int i=2;i<argc;i++) arg.push_back(argv[i]);
  if (argc == 2) {
    if (!((arg.front()=="--version") || (arg.front()=="--help"))) {
      arg.push_back("--help");
    }
  }
}

void tool::Parse() {
  LI i = arg.begin();
  while (i!=arg.end()) {
    string temp = *i;
    if (temp.substr(0,2)=="--") {
      char flag = temp[2];
      if (tool::OptionTags.find(flag)==string::npos) {
        (this->*tool::Options[UNKNOWN])(i);
      } else {
        (this->*tool::Options[flag])(i);
      }
    } else {
      i++;
    }
  }
  if (IsAKnownTool()) {
    AddSystemInfo();
  } else {
    if (!(helpfound || versionfound)) {
      if (arg.begin()!=arg.end()) {
        cerr << endl << "Error: win32fe: Unknown Tool: " << arg.front() << endl;
        cerr << "  Use --help for more information on win32fe options." << endl << endl;
      } else {
        cerr << endl << "Error: win32fe: Unknown Tool:" << endl;
        cerr << "  Use --help for more information on win32fe options." << endl << endl;
      }
    }
  }
}

void tool::AddSystemInfo(void) {
  if (detect) {
    FindInstallation();
    AddPaths();
  }
}

void tool::FindInstallation(void) {
  /* Find location of tool */
  InstallDir = arg.front();
  bool ierr = false;
  
    /* First check if a full path was specified with --use */
  string::size_type n = InstallDir.find(":");
  if (n==string::npos) {
    string extension = ".exe";
    char tmppath[MAX_PATH],*tmp; /* Windows.h */ 
    if (SearchPath(NULL,InstallDir.c_str(),extension.c_str(),MAX_PATH,tmppath,&tmp)) { /* Windows.h */
      /*
        Note: There are 6 locations searched, the 6th of these is the PATH variable.
        But, in reality, the first 5 don't make any sense for this tool, so it is 
        assumed that the tool was found in the PATH variable.
      */
      InstallDir = (string)tmppath;
    } else {
      ierr = true;
    }
  }
  string tool=arg.front();
  n = tool.rfind("\\")+1;
  tool = tool.substr(n,tool.rfind(".")-n);

  if (GetShortPath(InstallDir)) {
    inpath = true;
    n = InstallDir.rfind("\\BIN\\");
    if (n==string::npos) {
      n = InstallDir.rfind("\\Bin\\");
    }
    if (n==string::npos) {
      n = InstallDir.rfind("\\bin\\");
    }
    InstallDir = InstallDir.substr(0,n+1);
    if (verbose) {
      string longpath;
      GetLongPath(InstallDir,longpath);
      cout << "win32fe: " << tool << " Installation Found: " << longpath << endl;
    }
  } else {
    ierr = true;
  }
  if (ierr) {
    cerr << endl << "Error: win32fe: Tool Not Found: " << tool << endl;
    cerr << "  Specify the complete path to " << tool << " with --use" << endl;
    cerr << "  Use --help for more information on win32fe options." << endl << endl;
  }
}

void tool::AddPaths(void) {
  if (!inpath) {
    arg.push_back("--path");
    LI i = arg.end();
    i--;
    arg.push_back(InstallDir);
    FoundPath(i);
  }
}

void tool::Execute(void) {
  if (helpfound) {
    Help();
    return;
  }
  if (versionfound || verbose) {
    DisplayVersion();
  }
}

void tool::Help(void) {
  cout << endl << version_string << endl << endl;  
  cout << "Usage: win32fe <tool> --<win32fe options> -<tool options> <files>" << endl;
  cout << "  <tool> must be the first argument to win32fe" << endl << endl;
  cout << "<tool>: {cl,icl,df,f90,ifl,bcc32,lib,tlib}" << endl;
  cout << "  cl:    Microsoft 32-bit C/C++ Optimizing Compiler" << endl;
  cout << "  icl:   Intel C/C++ Optimizing Compiler" << endl;
  cout << "  df:    Compaq Visual Fortran Optimizing Compiler" << endl;
  cout << "  f90:   Compaq Visual Fortran90 Optimizing Compiler" << endl;
  cout << "  ifl:   Intel Fortran Optimizing Compiler" << endl;
  cout << "  bcc32: Borland C++ for Win32" << endl;
  cout << "  lib:   Microsoft Library Manager" << endl;
  cout << "  tlib:  Borland Library Manager" << endl << endl;
  cout << "<win32fe options>: {help,nodetect,path,use,verbose,version,woff}" << endl;
  cout << "  --help:       Output this help message and help for <tool>" << endl;
  cout << "  --nodetect:   Suppress automatic detection of <tool> installation" << endl;
  cout << "  --path <arg>: <arg> specifies an addition to the PATH that is required" << endl;
  cout << "                (ex. the location of a required .dll)" << endl;
  cout << "  --use <arg>:  <arg> specifies the variant of <tool> to use" << endl;
  cout << "  --verbose:    Echo to stdout the translated commandline" << endl;
  cout << "  --version:    Output version info for win32fe and <tool>" << endl;
  cout << "  --woff:       Suppress win32fe specific warning messages" << endl << endl;
  cout << "=========================================================================" << endl << endl;
}

void tool::FoundUnknown(LI &i) {
  i++;
}

void tool::FoundPath(LI &i) {
  if (*i=="--path") {
    i = arg.erase(i);
    if (i!=arg.end()) {
      /* Note 1024 might not be sufficiently large */
      int length = 1024;
      char buff[1024];
      string path="PATH";
      GetEnvironmentVariable(path.c_str(),buff,length); /* Windows.h */
      string newpath = (string)buff;

      string shortpath,inpath = *i;
      string::size_type n=inpath.find(";");
      string::size_type nold=0;
      do {
        string longpath,pathi=inpath.substr(nold,n);
        if (GetShortPath(pathi)) {
          if (verbose) {
            GetLongPath(pathi,longpath);
            cout << "win32fe: Adding to path: " << longpath << endl;
          }
          shortpath += pathi + ";";
        } else {
          if (!woff) {
            cout << "Warning: win32fe Path Not Found: " << inpath.substr(nold,n) << endl;
          }
        }
        nold = n;
        n = inpath.find(";",nold+1);
      } while (nold++!=string::npos);
      newpath = shortpath + newpath;
      SetEnvironmentVariable(path.c_str(),newpath.c_str()); /* Windows.h */
      i = arg.erase(i);
    } else {
      i--;
      arg.push_back("--help");
      i--;
    }
  }
}

void tool::FoundNoDetect(LI &i) {
  if (*i=="--nodetect") {
    detect = false;
    i = arg.erase(i);
    if (verbose) {
      cout << "win32fe: Turing off installation detection" << endl;
    }
  } else {
    i++;
  }
}

void tool::FoundUse(LI &i) {
  if (*i=="--use") {
    i = arg.erase(i);
    if (i!=arg.end()) {
      ReplaceSlashWithBackslash(*i);
      arg.pop_front();
      arg.push_front(*i);
      i = arg.erase(i);
    } else {
      i--;
      arg.push_back("--help");
      i--;
    }
  }
}

void tool::FoundWoff(LI &i) {
  if (*i == "--woff") {
    woff = true;
    i = arg.erase(i);
    cout << endl;
  } else {
    i++;
  }
}

void tool::FoundV(LI &i) {
  string temp = *i;
  if (temp == "--verbose") {
    verbose = true;
    i = arg.erase(i);
    cout << endl;
  } else if (temp == "--version") {
    versionfound=true;
    i = arg.erase(i);
  } else {
    i++;
  }
}

void tool::FoundHelp(LI &i) {
  if (*i == "--help") {
    helpfound = true;
    i = arg.erase(i);
  } else {
    i++;
  }
}

void tool::FoundFile(LI &i) {
  string temp = *i;
  ReplaceSlashWithBackslash(temp);
  /* If file specified with <path>\<filename> then verify it exists */
  string::size_type n = temp.rfind("\\");
  if (n!=string::npos) {
    string dir = temp.substr(0,n);
    if (GetShortPath(dir)) {
      temp = dir + temp.substr(n);
    } else {
      cerr << "Warning: win32fe: File Not Found: " << temp << endl;
    }
  }
  file.push_back(temp);
}

void tool::ReplaceSlashWithBackslash(string &name) {
  for (string::size_type i=0;i<name.length();i++)
    if (name[i]=='/') name[i]='\\';
}

void tool::ProtectQuotes(string &name) {
  string::size_type a,b;
  a = name.find("\"");
  if (a!=string::npos) {
    string temp = name.substr(0,a+1);
    temp += "\\\"";
    temp += name.substr(a+1,string::npos);
    name = temp;
    b = name.rfind("\"");
    if (b!=a+2) {
      temp = name.substr(0,b);
      temp += "\\\"";
      temp += name.substr(b,string::npos);
      name = temp;
    }
  }
}

#if defined(WINDOWS_NT4_OS)
/* GetShortPathName is broken on Win NT4 */
int tool::GetShortPath(string &shortpath) {
  string sofar,temp,tryme;
  string spath = shortpath;
  string::size_type relloc=0;

  /* If no drive letter is specified, path is relative */ 
  string::size_type oldloc=spath.find(":");
  if (oldloc==string::npos) {
    char dir[MAX_PATH]; /* Windows.h */
    GetCurrentDirectory(MAX_PATH,dir); /* Windows.h */
    string dirstr = (string)dir;
    /* If no leading \, path is relative to current directory */
    /* otherwise path is in current drive */
    if (spath[0]!='\\') {
      GetShortPath(dirstr); /* Is this necessary? */
      dirstr += "\\";
    } else {
      dirstr = dirstr.substr(0,dirstr.find(":")+2);
    }
    spath = dirstr + spath;
    /* Set relloc to be after the \ */
    relloc = dirstr.rfind("\\")+1;
    /* Set oldloc to be before the \ */
    oldloc = relloc-2;
  }
  oldloc += 1;
  string::size_type loc=oldloc+1;
  int    valid  = -1;

  /* So far, sofar is DriveLetter: (++ to avoid finding \ later) */
  sofar = spath.substr(0,oldloc++);
  /* FindFirstFile does not append the directory/drive info, so */
  /* we must search one directory at a time to get short dir names */
  /* as well as the short form of the file name.*/
  do {
    loc=spath.find("\\",oldloc);
    temp = spath.substr(oldloc,loc-oldloc);
    tryme = sofar + "\\" + temp;
    WIN32_FIND_DATA path; /* Windows.h */
    if (FindFirstFile(tryme.c_str(),&path) != INVALID_HANDLE_VALUE) { /* Windows.h */
      /* FindFirstFile sets path.cAlterhateFileName to the short name */
      /* ONLY if the short name exists (that is, ONLY if it is different from */
      /* the long name). */
      string altname = (string)path.cAlternateFileName; /* Windows.h */
      if (altname!="") {
        sofar += "\\" + altname;
      } else {
        sofar = tryme;
      }
    } else {
      valid=0;
      break;
    }
    oldloc = loc+1;
  } while (loc!=string::npos);
  if (valid) {
    shortpath = sofar.substr(relloc);
    valid = shortpath.length();
  }
  return(valid);
}

/* GetLongPathName does not exist on Win NT4 */
int tool::GetLongPath(string &shortpath,string &longpath) {
  string longtemp="";
  int    valid  = -1;

  string::size_type loc=shortpath.find(":");
  if (loc==string::npos) {
    loc=0;
  } else {
    loc+=2;
  }
  longtemp=shortpath.substr(0,loc);
  /* FindFirstFile does not append the directory/drive info, so */
  /* we must search one directory at a time to get long dir names */
  /* as well as the long form of the file name.*/
  while (loc!=string::npos) {
    loc=shortpath.find("\\",loc+1);
    WIN32_FIND_DATA path; /* Windows.h */
    if (INVALID_HANDLE_VALUE != FindFirstFile(shortpath.substr(0,loc).c_str(),&path)) { /* Windows.h */
      longtemp += (string)path.cFileName + "\\"; /* Windows.h */
    } else {
      valid=0;
      break;
    }
  }
  if (valid) {
    longpath = longtemp.substr(0,longtemp.length()-1);
    valid = longpath.length();
  } else {
    longpath = shortpath;
  }
  return(valid);
}

#else

int tool::GetShortPath(string &name) {
  /* Remove surrounding quotes */
  if (name[0]=='\''||name[0]=='\"')
    name=name.substr(1,name.length()-2);

  char shortpath[MAX_PATH]; /* Windows.h */
  int  size = GetShortPathName(name.c_str(),shortpath,MAX_PATH); /* Windows.h */
  /* size>=0 always. */ 
  /* if size > 0, then file exists and complete short path obtained */
  if (size) {
    name =(string)shortpath;
  }
  return(size);
}

int tool::GetLongPath(string &shortpath,string &longpath) {
  char longpath_c[MAX_PATH]; /* Windows.h */
  int  size = GetLongPathName(shortpath.c_str(),longpath_c,MAX_PATH); /* Windows.h */
  /* See GetShortPath */
  if (size) {
    longpath = (string)longpath_c;
  }
  return(size);
}
#endif

void tool::PrintListString(list<string> &liststr) {
  cout << "Printing..." << endl;
  LI i = liststr.begin();
  while (i!=liststr.end()) cout << *i++ << " ";
  cout << endl;
}

void tool::Merge(string &str,list<string> &liststr,LI &i) {
  while (i!=liststr.end()) {
    str += " " + *i++;
  }
}

void tool::DisplayVersion(void) {
  cout << version_string << endl;
}

bool tool::IsAKnownTool(void) {
  return(false);
}
